;;; compilation-moredirs.el --- more compilation-directory-matcher patterns

;; no need for regexp dup ?




;; Copyright 2009, 2010, 2011, 2014 Kevin Ryde

;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Version: 0 ...
;; Keywords: tools, processes, compilation
;; URL: http://user42.tuxfamily.org/compilation-moredirs/index.html

;; compilation-moredirs.el is free software; you can redistribute
;; it and/or modify it under the terms of the GNU General Public License as
;; published by the Free Software Foundation; either version 3, or (at your
;; option) any later version.
;;
;; compilation-moredirs.el is distributed in the hope that it will
;; be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You can get a copy of the GNU General Public License online at
;; <http://www.gnu.org/licenses/>.


;;; Commentary:
;; 

;;; Install:
;;
;;     (autoload 'compilation-moredirs-shell-cd "compilation-moredirs")
;;     (eval-after-load "compile" '(compilation-moredirs-shell-cd))

;;; History:
;; 

;;; Code:

(defun compilation-moredirs-add (group-count regexp &rest elements)
  ;; checkdoc-params: (group-count regexp elements)
  "This is an internal part of compilation-moredirs.el.
Add REGEXP and ELEMENTS to `compilation-directory-matcher', if
not already present.  Each element arg is a pair (DIR . LEAVE)
per that variable.

GROUP-COUNT is how many \\(\\) groups are in REGEXP, and thus how
much to offset existing elements in
`compilation-directory-matcher' when prepending REGEXP."

  (require 'compile)
  (cond ((boundp 'compilation-directory-matcher)
         (setq regexp (concat regexp "\\|"))
         (unless (let ((case-fold-search nil))
                   (string-match (concat "\\(\\`\\|\\\\|\\)" ;; start of string, or \|
                                         (regexp-quote regexp))
                                 (car compilation-directory-matcher)))
           (setq compilation-directory-matcher
                 (append (list (concat regexp
                                       (car compilation-directory-matcher)))
                         elements
                         (mapcar (lambda (elem)
                                   (cons (and (car elem)
                                              (+ (car elem) group-count))
                                         (and (cdr elem)
                                              (+ (cdr elem) group-count))))
                                 (cdr compilation-directory-matcher))))))

        ((boundp 'compilation-enter-directory-regexp-alist)
         ;; emacs20
         (add-to-list 'compilation-enter-directory-regexp-alist
                      (list regexp (caar elements))))))

;;;###autoload
(defun compilation-moredirs-shell-cd ()
  "Match \"cd\" directories in `compilation-mode'.
This function extends `compilation-directory-matcher' to match
lines like

    cd sub/dir                     - an echo
    + cd /some/where               - shell \"set -x\"
    echo hi; cd /tmp               \ combination echos
    cd /etc/emacs; cat foo         /

Plain \"cd\" is for a cd in a long `compile' command, inserted in
the buffer, or a diagnostic from your own script code.  \"+ cd\"
is for a shell script trace, see \"-x\" in Info node `(bash)The
Set Builtin'.

Relative directories are matched, but as of Emacs 23 they end up
relative to the compilation buffer directory, not a preceding
\"cd\" or other matched directory.  This means an initial
relative \"cd\" works, but after that it probably has to be an
absolute path.

There's nothing to notice when a subshell exits, returning to a
previously current directory."

  (interactive)
  (compilation-moredirs-add 1 "\\(?:^\\|; *\\|^\\++ \\)?cd \\([^ \t\n;]+\\)" '(1 . nil)))

;; LocalWords: dir Builtin subshell foo

(provide 'compilation-moredirs)

;;; compilation-moredirs.el ends here
