;;; xs-mode.el --- Perl XS editing mode

;; Copyright 2001, 2007, 2008, 2017 Kevin Ryde

;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Version: 0
;; Keywords: 
;; URL: http://user42.tuxfamily.org/zgrep/index.html

;; zgrep.el is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; zgrep.el is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You can get a copy of the GNU General Public License online at
;; <http://www.gnu.org/licenses/>.



;; not quite right



;;; Commentary:
;;
;; xs-mode is an editing mode for perl .xs file.  It's derived from c-mode,
;; but with some settings better suiting xs.

;; Install:
;;
;; To make M-x xs-mode available, put xs-mode.el in one of your `load-path'
;; directories and in your .emacs add
;;
;;	(autoload 'xs-mode "xs-mode" nil t)
;;
;; To use xs-mode automatically on appropriate files, put the following in
;; .emacs
;;
;;	(add-to-list 'auto-mode-alist '(".xs\\'" . xs-mode))
;;

;;; Code:

(require 'cc-mode)

;;;###autoload
(defgroup xs nil
  "XS mode."
  :prefix "xs-"
  :group 'languages)

(defcustom xs-mode-hook nil
  "Hook called by `xs-mode'."
  :type 'hook
  :group 'xs)


(defvar xs-mode-map
  (let ((map (make-sparse-keymap)))
    (set-keymap-parent map c-mode-map)
    ;;     (define-key map ";" 'self-insert-command)
    ;;     (define-key map "(" 'self-insert-command)
    ;;     (define-key map ")" 'self-insert-command)
    ;;     (define-key map "{" 'self-insert-command)
    ;;     (define-key map "}" 'self-insert-command)
    ;;     (define-key map ":" 'self-insert-command)
    ;;     (define-key map "*" 'self-insert-command)
    ;;     (define-key map "/" 'self-insert-command)
    map)
  "Keymap for `xs-mode'.")

(defconst xs-font-lock-keywords
  (eval-when-compile
    (list
     ;; Function names.
     '("^[A-Za-z0-9_]+\\s*("
       'font-lock-function-name-face)
     ;;
     ;; Variable names.
     (cons
      (concat "\\<"
              (regexp-opt
               '("CODE:" "PPCODE:" "INIT:" "NO_INIT" "PREINIT:"
                 "OUTPUT:" "NO_OUTPUT" "SCOPE:" "INPUT:"
                 "C_ARGS:" "REQUIRE:" "CLEANUP:" "POSTCALL:" "BOOT:"
                 "VERSIONCHECK:" "PROTOTYPE:" "PROTOTYPES:" "ALIAS:"
                 "OVERLOAD:" "FALLBACK:" "INTERFACE:" "INTERFACE_MACRO:"
                 "INCLUDE:" "CASE:") t) "\\>")
      'font-lock-keyword-face))))

(defun xs-beginning-of-defun (&optional count)
  (re-search-backward "^[A-Za-z0-9_]+\\s*(") nil nil count)

(c-add-style "XS" '("GNU"
                    (c-offsets-alist
                     (topmost-intro . 4)
                     (knr-argdecl-intro . 6)
                     (label . xs-mode-keyword)
                     )))

;;;###autoload
(defun xs-mode ()
  "Major mode for editing C based perl .xs files.
Derived from `c-mode'.

\\{xs-mode-map}

\\{c-mode-map}"

  (interactive)
  (c-mode)
  ;;  (setq major-mode 'c-mode
  ;;        mode-name  "C/XS")

  (setq paragraph-separate (concat paragraph-separate "\\|^=[a-z]"))
  (setq beginning-of-defun-function 'xs-beginning-of-defun)

  (set (make-local-variable 'require-final-newline) t)

  (use-local-map xs-mode-map)

  ;;   (c-set-style "gnu")
  ;;
  ;;   (c-add-style
  ;;    "XGNU"
  ;;    '((c-indent-level . 2)
  ;;      (c-argdecl-indent . 5)
  ;;      (c-brace-offset . 0)
  ;;      (c-continued-brace-offset . 0)
  ;;      (c-label-offset . -2)
  ;;      (c-continued-statement-offset . 2)))

  (if (eval-when-compile (fboundp 'run-mode-hooks))
      (run-mode-hooks 'xs-mode-hook)
    (run-hooks 'xs-mode-hook)))


(defun xs-mode-pos-column (pos)
  (save-excursion
    (goto-char pos)
    (current-column)))

(defun xs-mode-keyword (s)
  (message "at %d" (point))
  (if (looking-at "CODE:")
      (- (xs-mode-pos-column (cdr s)))
    0))
  

(provide 'xs-mode)

;;; xs-mode.el ends here
