#!/bin/sh

# Copyright 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016 Kevin Ryde.
#
# This file is part of miscbits-el.
#
# miscbits-el is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# miscbits-el is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with miscbits-el; see the file COPYING.  Failing that, go to
# <http://www.gnu.org/licenses/>.

set -x

SITE_LISP=/so/emacs20/emacs-20.7/inst/share/emacs/site-lisp

# /usr/share/emacs/site-lisp/apel/poe.el

#  ac-tex-ref.el                -- think auto-complete not for e20
#  ac-tex-macros-used.el        -- think auto-complete not for e20
#  checkdoc-batch.el            -- replace-regexp-in-string
#  compilation-always-kill.el	-- define-minor-mode
#  compilation-onewin.el	-- define-minor-mode, make-hash-table
#  completion-ignored-build.el	-- define-minor-mode
#  copyright-both-ends.el       -- no copyright.el
#  emacs-lisp-docstring-preview.el  -- with-selected-window
#  formfeed-hline.el		-- define-minor-mode
#  ffap-makefile-vars.el        -- replace-regexp-in-string
#  help-xref-elfiles.el         -- no define-button-type so does nothing
#  gutenberg-coding.el		-- replace-regexp-in-string, #hex syntax
#  html-coding.el
#  html-sizes.el		-- perform-replace args
#  imagetext.el			-- bindat.el, #hex syntax
#  info-lookmore.el		-- add-to-list append
#  localedef-overlay.el         -- define-minor-mode
#  man-completion.el            -- replace-regexp-in-string
#  mo-mode.el			-- make-temp-file
#  nroff-filladapt.el		-- add-to-list append
#  oeis.el                      -- partly works, time bits, replace-in-string
#  parasep.el			-- partly works, split-string keep empties
#  perl-use-utf8-coding.el
#  perl-pod-coding.el
#  perl-pod-gt.el               -- partly works
#  perl-pod-preview.el		-- make-temp-file
#  sh-autoconf.el		-- add-to-list append
#  sentence-spacing.el		-- regexp [:alnum:] classes
#  texinfo-warn.el		-- define-minor-mode
#  tex-increase.el		-- derived-mode-p
#  tex-math-preview.el		-- derived-mode-p
#  tty-format.el		-- ok with ansi-color.el from elsewhere
#  unicode-disp.el              -- no unicode unless maybe mule-ucs
#                                  no decode-char, no #xFF literals
#                                  would want 'cl for dolist
#  xml-coding.el
#

EL_FILES_COMPILE_NOT="
  ac-tex-ref.el
  ac-tex-macros-used.el
  emacs-lisp-docstring-preview.el
  checkdoc-batch.el
  compilation-always-kill.el
  compilation-onewin.el
  completion-ignored-build.el
  copyright-both-ends.el
  formfeed-hline.el
  ffap-makefile-vars.el
  gutenberg-coding.el
  html-coding.el
  html-sizes.el
  imagetext.el
  localedef-overlay.el
  perl-use-utf8-coding.el
  perl-pod-coding.el
  sentence-spacing.el
  sh-autoconf.el
  tex-increase.el
  texinfo-warn.el
  info-lookmore.el
  mo-mode.el
  nroff-filladapt.el
  perl-pod-preview.el
  unicode-disp.el
  xml-coding.el
"

EL_FILES_COMPILE="
  xtide.el
  tcd-format.el
  align-let.el
  arc-lzh-exe.el
  areabrowse.el
  bytecomp-simplify.el
  checkdoc-autoload.el
  checkdoc-keywords.el
  checkdoc-url-buttonize.el
  checkdoc-vs-flyspell.el
  compilation-emacs21-batch.el
  compilation-ledger.el
  compilation-perl.el
  compilation-recenter-end.el
  compile-command-default.el
  compile-history-local.el
  cl-loop-aplist.el
  diff-save-buffer.el
  dired-visit-history.el
  do-not-edit.el
  elisp-docstring-preview.el
  filladapt-pat.el
  ffap-gcc-path.el
  ffap-href.el
  ffap-I-option.el
  ffap-include-start.el
  ffap-mml.el
  ffap-perl-module.el
  ffap-pod-F.el
  ffap-rfc-directories.el
  ffap-rfc-space.el
  gtk-look.el
  help-xref-elfiles.el
  ispell-stopwords.el
  jka-compr-dictzip.el
  jka-compr-lz4.el
  makeinfo-info.el
  make-mode-fillindent.el
  mailcap-mode.el
  man-completion.el
  man-preview.el
  mine-sweeper-auto.el
  nobreak-fade.el
  oeis.el
  parasep.el
  perl-pod-gt.el
  perl-quote.el
  perl-env-substitute.el
  pst-format.el
  quick-yes.el
  renice.el
  scroll-1.el
  sqlite-dump.el
  startup-file-name-history.el
  texinfo-nobreak.el
  tex-math-preview.el
  tty-format.el
  variable-at-point-tick.el
  upstr.el
  wx-look.el
"
EL_FILES="$EL_FILES_COMPILE"
# miscbits-el-finder-inf.el

duplicates=`(for i in $EL_FILES_COMPILE $EL_FILES_COMPILE_NOT; do echo $i; done) | sort | uniq -d`
if test -n "$duplicates"; then
  echo "Duplicate COMPILE and COMPILE_NOT: $duplicates";
  exit 1;
fi

cd ..
TOPDIR="`pwd`"
TOOLSDIR="$TOPDIR/tools"

cp /down/el/emacs20/ansi-color.el \
  $SITE_LISP/ansi-color.el
emacs20 -q -no-site-file -batch \
  -l $TOOLSDIR/miscbits-byte-compile.el \
  -f batch-byte-compile \
  $SITE_LISP/ansi-color.el

cd $SITE_LISP
emacs20 -q -no-site-file -batch \
  -l $TOOLSDIR/miscbits-generate-loaddefs.el \
  -f my-loaddefs-batch \
  $EL_FILES
# mv miscbits-el-loaddefs.el $SITE_LISP

emacs20 -q -no-site-file -batch \
  -l $TOOLSDIR/miscbits-byte-compile.el \
  -f batch-byte-compile \
  miscbits-el-loaddefs.el

cd $SITE_LISP
rm -f $EL_FILES

cd $TOPDIR
for i in $EL_FILES; do
  FROMDIR=
  if [ -e ../tcd/$i ];      then FROMDIR=../tcd/;      fi
  if [ -e ../xtide/$i ];    then FROMDIR=../xtide/;    fi
  if [ -e wip/$i ];         then FROMDIR=wip/;         fi
  if [ -e coming-soon/$i ]; then FROMDIR=coming-soon/; fi
  if [ -e $HOME/lisp/$i ];  then FROMDIR=$HOME/lisp/;  fi
  cp -f $FROMDIR$i $SITE_LISP/$i
  chmod 444 $SITE_LISP/$i
done

e=`(cd $TOPDIR; ls *.el; cd $TOPDIR/../xtide; ls *.el; cd $TOPDIR/../tcd; ls *.el)`
e=`(for i in $e $EL_FILES_COMPILE_NOT; do echo $i; done) | tee /dev/stderr|sort | uniq -u`
echo $e
cd $SITE_LISP
for i in $e; do
  if ! test -e $i; then
    echo "New further file: $i";
    exit 1;
  fi
done

cd $SITE_LISP
for i in $EL_FILES; do
  echo $i;
  emacs20 -q -no-site-file -batch \
    -l $TOOLSDIR/miscbits-byte-compile.el \
    -f batch-byte-compile \
    $i
done
