#!/usr/bin/perl -w

# Copyright 2009, 2010, 2011 Kevin Ryde.

# This file is part of miscbits-el.
#
# miscbits-el is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# miscbits-el is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with miscbits-el; see the file COPYING.  Failing that, go to
# <http://www.gnu.org/licenses/>.


use strict;
use warnings;
use POSIX;
use Perl6::Slurp;
use File::Find::Rule;

use lib::abs '.';
use MyStuff;
use MyUniqByInode;

my $verbose = 0;

my $exclude_dirs_re = qr{(^|/)(blib|CVS|\.svn)$};
# emacs backup foo~
# emacs lock .#foo or autosave #foo#
my $exclude_files_re = qr/~$|^\.?#/;

my $rule = File::Find::Rule->or
  (File::Find::Rule->directory->name($exclude_dirs_re)->prune->discard,
   File::Find::Rule->directory->discard,
   File::Find::Rule->file->name($exclude_files_re)->discard,
   File::Find::Rule->name('*.el'));
$rule->start ('/down/emacs');

my $uniq = MyUniqByInode->new;

while (my $filename = $rule->match) {
  if ($verbose) { print "look at $filename\n"; }

  $uniq->uniq($filename) || next;

  my $str;
  if (! eval { $str = Perl6::Slurp::slurp($filename); 1 }) {
    unless ($! == ENOENT) {
      print "Cannot read $filename: $!\n";
    }
    next;
  }

  # reftex.el and org-table.el `:included' for add-minor-mode menu entry
  # next unless ($str =~ /put.*:included/mg);

  #   next unless ($str =~ /\n\n\(define-minor-mode/g);

  # next unless ($str =~ /\n\n\(define-minor-mode(.*\n){0,50}.*:included/mg);
  #  next unless ($str =~ /\n\n\(define-minor-mode.*:included/g);

  # non-autoload define-minor-mode
  #
  $str =~ /(?<!\n;;;###autoload)\n\(define-minor-mode ([^ \n]*)/g
    or next;
  my $pos = $-[1];
  my $found = $1;

  my ($line, $col) = MyStuff::pos_to_line_and_column ($str, $pos);
  print "$filename:$line:$col: found $found\n";
  #  print "$found\n";
}

exit 0;


__END__

(pp (let (ret)
     (mapatoms (lambda (sym)
                (and (get sym :included)
                 (push (list sym (get sym :included)) ret))))
     ret)
 (current-buffer))


(progn
  (require 'cl)
  (remove-if 'fboundp
             '(abbrev-mode
               diff-auto-refine-mode
               dired-omit-mode
               dirtrack-debug-mode
               blink-cursor-mode
               temp-buffer-resize-mode
               next-error-follow-minor-mode
               menu-bar-mode
               mouse-wheel-mode
               cvs-minor-mode
               file-name-shadow-mode
               tool-bar-mode
               shell-dirtrack-mode
               tooltip-mode
               vcursor-use-vcursor-map
               text-scale-mode
               font-lock-mode
               ido-everywhere
               auto-compression-mode
               auto-encryption-mode
               gnus-message-citation-mode
               sc-minor-mode
               rcirc-multiline-minor-mode
               xesam-minor-mode
               unify-8859-on-encoding-mode
               thai-word-mode
               idlwave-shell-electric-debug-mode
               pascal-outline-mode
               use-hard-newlines
               sgml-electric-tag-pair-mode
               nroff-electric-mode
               ede-minor-mode
               semantic-idle-summary-mode
               org-cdlatex-mode
               org-remember-mode
               org-src-mode)))
