;; Copyright 2007, 2008, 2009, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  ;; (require 'perl-quote)

  (let ((count 0)
        (case-fold-search t)
        (tempbuf "*t-perl-quote.el temp*"))
    (switch-to-buffer tempbuf)
    (erase-buffer)
    (cperl-mode)

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(perl-quote-single
              perl-quote-double))

    ;;-------------------------------------------------------------------------
    ;; perl-quote-single

    (mapcar (lambda (elem)
              (let ((str  (nth 0 elem))
                    (want (nth 1 elem))
                    want-point)
                (switch-to-buffer "*t-perl-quote.el temp*")
                (erase-buffer)
                (insert str)
                (cperl-update-syntaxification (point-min) (point-max))
                (goto-char (point-min))
                (setq want-point (point))
                (let* ((got (condition-case err
                                (progn
                                  (perl-quote-single)
                                  (buffer-string))
                              (error
                               (setq want-point (point))
                               (error-message-string err))))
                       (got-point (point)))
                  (unless (equal got want)
                    (goto-char (point-max))
                    (insert (format "\nperl-quote-single\nstr:  %s\nwant: %s\ngot: %s"
                                    str want got))
                    (error "got %s want %s" got want))
                  (unless (= got-point want-point)
                    (error "point moved"))))
              (setq count (1+ count)))

            '(("\"foo\""       "'foo'")
              ("\"fo \\\\ o\"" "'fo \\\\ o'")
              ("\"foo\\\\t\"" "'foo\\\\t'")
              ("\"fo\\no\""    "'fo\no'")
              ("\"fo\\t\\to\"" "'fo\t\to'")
              ("\"fo''o\""     "'fo\\'\\'o'")
              ("\"fo\\\"o\""   "'fo\"o'")

              ("__\"foo\""     "__('foo')")
              ("N__\"foo\""    "N__('foo')")

              ("qq<foo>"       "q<foo>")
              ("qq<\\tfoo>"    "q<\tfoo>")
              ("qq[foo\\t]"    "q[foo\t]")
              ("qq{\\\"}"      "q{\"}")

              ("qq{$var}"      "Interpolation in string")
              ("qq{@array}"    "Interpolation in string")
              ("qq{\\$}"       "q{$}")
              ("qq{\\@}"       "q{@}")
              ))

    ;;-------------------------------------------------------------------------
    ;; perl-quote-double

    (mapcar (lambda (elem)
              (let ((str  (car elem))
                    (want (cadr elem)))
                (switch-to-buffer "*t-perl-quote.el temp*")
                (erase-buffer)
                (insert str)
                (cperl-update-syntaxification (point-min) (point-max))
                (goto-char (point-min))
                (perl-quote-double)
                (let ((got (buffer-string)))
                  (unless (equal got want)
                    (goto-char (point-max))
                    (insert (format "\nperl-quote-double\nstr:  %s\nwant: %s\ngot:  %s\n"
                                    str want got))
                    (error "got %s want %s" got want))
                  (unless (= (point) (point-min))
                    (error "point moved"))))
              (setq count (1+ count)))

            '(("'foo'"       "\"foo\"")
              ("'fo\\'o'"    "\"fo'o\"")
              ("'fo\\'o\\''" "\"fo'o'\"")
              ("'fo\\\\o'"   "\"fo\\\\o\"")
              ("'fo\"o'"     "\"fo\\\"o\"")

              ("q<foo>"      "qq<foo>")
              ("q{'}"        "qq{'}")
              ("q{\"}"       "qq{\"}")

              ;; cperl-mode in emacs21 doesn't get '$' right
              ("'$x'"        "\"\\$x\"")
              ("'@'"         "\"\\@\"")
              ("'\"'"        "\"\\\"\"")

              ("'\\$'"       "\"\\$\"")
              ("'\\@'"       "\"\\@\"")
              ("'\\\"'"      "\"\\\"\"")

              ;; not sure about this, \" is not a defined escape in singles
              ;; ("q{\\\"}"     "qq{\\\\\"}")
              ))


    (bury-buffer)
    (message "  %d tests" count)
    count))
